/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Context Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Context.h,v 1.3 2006/11/28 12:01:16 badino Exp $
 *  Release: $Name: glite-data-srm-util-cpp_R_1_1_0_1 $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_CONTEXT_H_
#define GLITE_DATA_SRM_UTIL_CPP_CONTEXT_H_

#include <string>
#include <boost/shared_ptr.hpp>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {

/**
 * SRM Context Class
 */
struct Context{
    
    /**
     * The Context Pointer type
     */
    typedef boost::shared_ptr<Context> Ptr;
    
    /**
     * The Hook for the SRM calls
     */
    typedef void (*Hook)(Context& ctx, const char * srm_method);

    /**
     * The Hook for triggering come Warnings
     */
    typedef void (*WarnHook)(Context& ctx, const char * scope, const char * description);
    
    /**
     * SRM endpoint to be used
     */
    std::string endpoint;
    /**
     * SRM interface version
     */
    std::string version;
    /**
     * Timeout for HTTP calls
     */
    unsigned long httpTimeout;
    
    /**
     * If some features are missing in the required SRM interface version,
     * ignore them, otherwise throw an exception
     */
    bool ignoreMissingFeatures;

    /**
     * Hook triggered before sending a SOAP message
     */
    Hook pre_call;
    
    /**
     * Hook triggered just after the SOAP method is executed
     */
    Hook post_call_success;

    /**
     * Hook triggered just after the SOAP method failed
     */
    Hook post_call_failure;

    /**
     * Hook triggered when a warning is raised (like an ignored/handled exception)
     */
    WarnHook warn;

    /**
     * Constructor
     */    
    Context():
        httpTimeout(60),ignoreMissingFeatures(true),
        pre_call(0),post_call_success(0),post_call_failure(){}
            
    /**
     * call the pre_call hook
     */            
    void beforeCall(const char * method) throw() {
        try{
            if(0 != this->pre_call){
                (*(this->pre_call))(*this,method);
            }
        } catch(...){
            // Ignore
        }
    }
    
    /**
     * call the post_call_success hook
     */            
    void onSuccess(const char * method) throw() {
        try{
            if(0 != this->post_call_success){
                (*(this->post_call_success))(*this,method);
            }
        } catch(...){
            // Ignore
        }
    }
    
    /**
     * call the post_call_failed hook
     */            
    void onFailure(const char * method) throw() {
        try{
            if(0 != this->post_call_failure){
                (*(this->post_call_failure))(*this,method);
            }
        } catch(...){
            // Ignore
        }
    }
    
    /**
     * call the warn hook
     */            
    void onWarning(const char * scope,const char * description) throw() {
        try{
            if(0 != this->warn){
                (*(this->warn))(*this,scope,description);
            }
        } catch(...){
            // Ignore
        }
    }
};


} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_CONTEXT_H_
